/*
 * ConnectionDialog.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.jobexftp;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.HashMap;
import java.util.Map;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.underworldlabs.jobexftp.util.GUIUtilities;
import org.underworldlabs.jobexftp.util.ObexWorker;
import org.underworldlabs.jobexftp.util.SystemResources;
import org.underworldlabs.swing.ActionPanel;
import org.underworldlabs.swing.GUIUtils;
import org.underworldlabs.swing.util.SwingWorker;
import org.underworldlabs.util.MiscUtils;
import org.underworldlabs.util.SystemProperties;

/**
 *
 * @author Takis Diakoumis
 */
public class ConnectionDialog extends JDialog 
                              implements InterruptibleProcess,
                                         ActionListener {
    
    /** the process worker utility */
    private ObexWorker worker;

    /** swing thread worker */
    private SwingWorker swingWorker;
    
    /** progress dialog */
    private ObexProcessProgressDialog progressDialog;

    /** the connect button */
    private JButton connectButton;

    /** the cancel button */
    private JButton cancelButton;

    /** options base panel */
    private BasePanel contentPanel;
    
    /** Creates a new instance of ConnectionDialog */
    public ConnectionDialog() {
        super(GUIUtilities.getApplicationFrame(), "Connection", true);
        try {
            init();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    private void init() throws Exception {
        connectButton = new JButton("Connect");
        connectButton.addActionListener(this);

        cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(this);

        Dimension btnDim = new Dimension(100,25);
        cancelButton.setPreferredSize(btnDim);
        connectButton.setPreferredSize(btnDim);

        contentPanel = new BasePanel();
        contentPanel.setBorder(BorderFactory.createEmptyBorder(10,5,5,0));
        
        JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        buttonPanel.add(connectButton);
        buttonPanel.add(cancelButton);
        buttonPanel.setBorder(BorderFactory.createEmptyBorder(0,0,5,5));
        
        JPanel base = new JPanel(new BorderLayout());
        base.add(contentPanel, BorderLayout.CENTER);
        base.add(buttonPanel, BorderLayout.SOUTH);

        base.setBorder(BorderFactory.createEtchedBorder());
        base.setPreferredSize(new Dimension(500,220));
        
        Container c = this.getContentPane();
        c.setLayout(new GridBagLayout());
        c.add(base, new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0,
                                           GridBagConstraints.SOUTHEAST, 
                                           GridBagConstraints.BOTH,
                                           new Insets(5, 5, 5, 5), 0, 0));
        
        setResizable(false);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        
        pack();
        setLocation(GUIUtilities.getLocationForDialog(getSize()));
        setVisible(true);

    }

    /**
     * Button action implementation.
     */
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == connectButton) {
            String[] args = contentPanel.getConnectionArgs();
            if (args != null) {
                connect(args);
            }
        } 
        else if (source == cancelButton) {
            dispose();
        }
    }

    public void cancel() {
        try {
            if (swingWorker != null) {
                swingWorker.interrupt();
            }
        }
        finally {
            if (worker != null && worker.isProcessing()) {
                worker.destroyRunningProcess();
            }
        }
    }

    private void connect(final String[]  args) {
        if (worker == null) {
            worker = new ObexWorker();
        }

        swingWorker = new SwingWorker() {
            public Object construct() {
                GUIUtilities.showWaitCursor();
                setVisible(false);
                try {
                    return worker.connect(contentPanel.getConnectionType(),args);
                }
                catch (Exception e) {
                    e.printStackTrace();
                    return e;
                }
            }
            public void finished() {
                if (progressDialog != null) {
                    progressDialog.dispose();
                }
                GUIUtilities.showNormalCursor();
                Object object = get();

                if (object instanceof Integer) {
                    Integer result = (Integer)get();

                    int _result = result.intValue();
                    if (_result == Constants.CONNECTED) {
                        // set the the user props
                        contentPanel.setUserProperties();

                        SystemResources.saveUserProperties(
                                SystemProperties.getProperties("user"));
                        
                        // inform the controller
                        JObexController.setConnected(true);

                    }
                    else if (_result == Constants.PROCESS_CANCELLED) {
                        GUIUtilities.displayWarningMessage(
                                "Connection cancelled at user request");
                    }
                    dispose();

                }
                // case of dump on connect
                else if (object instanceof Throwable) {
                    GUIUtilities.displayExceptionErrorDialog(
                            "Error establishing connection to device.", 
                            (Throwable)object);
                    setVisible(true);
                }

                GUIUtils.scheduleGC();
            }
        };
        progressDialog = new ObexProcessProgressDialog(
                this, "Connecting...", "Connecting to device...");
        swingWorker.start();
        progressDialog.run();
    }

    
    class BasePanel extends ActionPanel 
                    implements ItemListener {
        
        /** the connection type combo box */
        private JComboBox connTypeCombo;
        
        /** the base panel for selection options */
        private JPanel optionsPanel;
        
        /** the options panel layout */
        private CardLayout cardLayout;
        
        /** the obexftp exe location field */
        private JTextField obexftpField;

        /** the bluetooth address field */
        private JComboBox bluetoothAddressField;

        /** the bluetooth channel field */
        private JTextField bluetoothChannelField;

        /** the usb interface field */
        private JTextField usbInterfaceField;

        /** the serial device name field */
        private JTextField serialDeviceField;
        
        /** the network host field */
        private JTextField networkHostField;
        
        /** options panel map cache */
        private Map<String,JPanel> optionsPanels;
        
        public BasePanel() {
            super(new GridBagLayout());
            
            String[] connectionTypes = {"IrDA", "Bluetooth", "USB",
                                        "Serial (tty)", "Network"};
            
            connTypeCombo = new JComboBox(connectionTypes);
            connTypeCombo.addItemListener(this);
            
            // initialise the oberftp exe location field
            obexftpField = new JTextField();
            // set the value from the user prefs
            obexftpField.setText(
                    SystemProperties.getProperty("user", "obexftp.executable"));

            
            // initialise the card options panel
            cardLayout = new CardLayout();
            optionsPanel = new JPanel(cardLayout);

            // initialise the panel cache for the layout
            optionsPanels = new HashMap<String,JPanel>();
            
            GridBagConstraints gbc = new GridBagConstraints();
            gbc.anchor = GridBagConstraints.NORTHWEST;
            gbc.gridy = 0;
            gbc.insets.right = 10;
            add(new JLabel("ObexFTP executable:"), gbc);
            gbc.gridx = 1;
            gbc.weightx = 1.0;
            gbc.fill = GridBagConstraints.HORIZONTAL;
            add(obexftpField, gbc);
            gbc.gridy++;
            gbc.gridx = 0;
            gbc.weightx = 0;
            gbc.insets.top = 8;
            gbc.fill = GridBagConstraints.NONE;
            add(new JLabel("Connection type:"), gbc);
            gbc.gridx = 1;
            gbc.weightx = 1.0;
            gbc.insets.top = 5;
            gbc.fill = GridBagConstraints.HORIZONTAL;
            add(connTypeCombo, gbc);
            gbc.gridx = 0;
            gbc.gridy++;
            gbc.weightx = 1.0;
            gbc.weighty = 1.0;
            gbc.insets.top = 15;
            gbc.fill = GridBagConstraints.BOTH;
            gbc.gridwidth = GridBagConstraints.REMAINDER;
            gbc.gridheight = GridBagConstraints.REMAINDER;
            add(optionsPanel, gbc);
            
            // set the last user selection if available
            String value = SystemProperties.getProperty("user", "obex.connection.type");
            if (!MiscUtils.isNull(value)) {
                int _value = Integer.parseInt(value);
                if (_value >= 0 && _value < connectionTypes.length) {
                    connTypeCombo.setSelectedIndex(_value);
                }
            }

        }

        /**
         * Sets respective user properties based on field entries.
         */
        protected void setUserProperties() {
            // set the executable path
            String value = obexftpField.getText();
            SystemProperties.setProperty("user", "obexftp.executable", value);
            
            int index = connTypeCombo.getSelectedIndex();
            
            SystemProperties.setIntProperty(
                    "user", "obex.connection.type", index);

            switch (index) {
                case Constants.BLUETOOTH_CONNECTION:
                    // check the address field
                    value = bluetoothAddressField.getEditor().getItem().toString();
                    if (!MiscUtils.isNull(value)) {
                        
                        String addresses = SystemProperties.getProperty(
                                "user", "obex.bluetooth.address");

                        if (!MiscUtils.isNull(addresses)) {                            
                            if (!addresses.contains(value)) {
                                addresses += "," + value;
                            }
                        } else {
                            addresses = value;
                        }

                        SystemProperties.setProperty(
                                "user", "obex.bluetooth.address", addresses);
                        
//                        SystemProperties.setProperty(
//                                "user", "obex.bluetooth.address", value);
                    } 

                    // check the channel field
                    value = bluetoothChannelField.getText();
                    if (!MiscUtils.isNull(value)) {
                        SystemProperties.setProperty(
                                "user", "obex.bluetooth.channel", value);
                    }
                    break;

                case Constants.USB_CONNECTION:
                    // check the interface field
                    value = usbInterfaceField.getText();
                    if (!MiscUtils.isNull(value)) {
                        SystemProperties.setProperty(
                                "user", "obex.usb.interface", value);
                    }
                    break;

                case Constants.SERIAL_CONNECTION:
                    // check the interface field
                    value = usbInterfaceField.getText();
                    if (!MiscUtils.isNull(value)) {
                        SystemProperties.setProperty(
                                "user", "obex.serial.device", value);
                    }
                    break;

                case Constants.NETWORK_CONNECTION:
                    // check the interface field
                    value = usbInterfaceField.getText();
                    if (!MiscUtils.isNull(value)) {
                        SystemProperties.setProperty(
                                "user", "obex.network.host", value);
                    }
                    break;

            }
            
        }
        
        /**
         * Returns the connection type.
         */
        protected int getConnectionType() {
            return connTypeCombo.getSelectedIndex();
        }
        
        /**
         * Returns the connection arguments from user input.
         */
        protected String[] getConnectionArgs() {
            String value = obexftpField.getText();
            if (MiscUtils.isNull(value)) {
                GUIUtilities.displayErrorMessage(
                        "You must provide a valid path to the oberftp command");
                return null;
            }

            String[] args = null;
            int index = connTypeCombo.getSelectedIndex();
            
            switch (index) {
                case Constants.IRDA_CONNECTION:
                    return new String[0];

                case Constants.BLUETOOTH_CONNECTION:
                    
                    // 2 possible arguments for bluetooth
                    args = new String[2];
                    
                    // check the address field
                    //value = bluetoothAddressField.getText();
                    value = bluetoothAddressField.getEditor().getItem().toString();
                    if (!MiscUtils.isNull(value)) {
                        args[0] = value;
                    }

                    // check the channel field
                    value = bluetoothChannelField.getText();
                    if (!MiscUtils.isNull(value)) {
                        args[1] = value;
                    } 
                    return args;

                case Constants.USB_CONNECTION:

                    // 1 possible arguments for usb
                    args = new String[1];

                    // check the interface field
                    value = usbInterfaceField.getText();
                    if (!MiscUtils.isNull(value)) {
                        args[0] = value;
                    }
                    return args;

                case Constants.SERIAL_CONNECTION:

                    // 1 possible arguments for serial
                    args = new String[1];

                    // check the interface field
                    value = serialDeviceField.getText();
                    if (!MiscUtils.isNull(value)) {
                        args[0] = value;
                    }
                    return args;

                case Constants.NETWORK_CONNECTION:
                    
                    // 1 possible arguments for serial
                    args = new String[1];

                    // check the interface field
                    value = networkHostField.getText();
                    if (!MiscUtils.isNull(value)) {
                        args[0] = value;
                    }
                    return args;

            }
            return new String[0];
        }
        
        /**
         * Switches the option panel currently displayed.
         *
         * @param name - the layout name to switch to
         * @param panel - the panel to switch to
         */
        private void optionChanged(String name, JPanel panel) {
            optionsPanel.add(panel, name);
            cardLayout.show(optionsPanel, name);
            optionsPanel.repaint();
        }
        
        /**
         * Invoked when an connection type has been selected 
         * or deselected by the user.
         */    
        public void itemStateChanged(ItemEvent e) {
            int index = connTypeCombo.getSelectedIndex();
            switch (index) {
                case Constants.IRDA_CONNECTION:
                    optionChanged("irda", getIrdaOptionsPanel());
                    break;
                case Constants.BLUETOOTH_CONNECTION:
                    optionChanged("bluetooth", getBluetoothOptionsPanel());
                    break;
                case Constants.USB_CONNECTION:
                    optionChanged("usb", getUsbOptionsPanel());
                    break;
                case Constants.SERIAL_CONNECTION:
                    optionChanged("serial", getSerialOptionsPanel());
                    break;
                case Constants.NETWORK_CONNECTION:
                    optionChanged("network", getNetworkOptionsPanel());
                    break;
                
            }
        }

        /**
         * Builds the Serial options panel.
         */
        private JPanel getSerialOptionsPanel() {
            if (optionsPanels.containsKey("serial")) {
                return optionsPanels.get("serial");
            }
            
            serialDeviceField = new JTextField();

            JPanel panel = new JPanel(new GridBagLayout());
            GridBagConstraints gbc = new GridBagConstraints();
            gbc.anchor = GridBagConstraints.NORTHWEST;
            gbc.gridy = 0;
            gbc.insets.left = 5;
            gbc.insets.right = 5;
            gbc.insets.top = 5;
            panel.add(new JLabel("Device (/dev/ttySxx):"), gbc);
            gbc.gridx = 1;
            gbc.weighty = 1.0;
            gbc.weightx = 1.0;
            gbc.fill = GridBagConstraints.HORIZONTAL;
            panel.add(serialDeviceField, gbc);

            panel.setBorder(BorderFactory.createTitledBorder("Options"));

            // set the value from the user prefs
            serialDeviceField.setText(
                    SystemProperties.getProperty("user", "obex.serial.device"));

            
            optionsPanels.put("serial", panel);
            return panel;
        }

        /**
         * Builds the irda options panel.
         */
        private JPanel getIrdaOptionsPanel() {
            if (optionsPanels.containsKey("irda")) {
                return optionsPanels.get("irda");
            }

            JPanel panel = new JPanel();//new FlowLayout(FlowLayout.LEFT));
            //panel.add(new JLabel("Not Available"));
            optionsPanels.put("irda", panel);
            return panel;
        }

        /**
         * Builds the network options panel.
         */
        private JPanel getNetworkOptionsPanel() {
            if (optionsPanels.containsKey("network")) {
                return optionsPanels.get("network");
            }

            networkHostField = new JTextField();

            JPanel panel = new JPanel(new GridBagLayout());
            GridBagConstraints gbc = new GridBagConstraints();
            gbc.anchor = GridBagConstraints.NORTHWEST;
            gbc.gridy = 0;
            gbc.insets.left = 5;
            gbc.insets.right = 5;
            gbc.insets.top = 5;
            panel.add(new JLabel("Host:"), gbc);
            gbc.gridx = 1;
            gbc.weighty = 1.0;
            gbc.weightx = 1.0;
            gbc.fill = GridBagConstraints.HORIZONTAL;
            panel.add(networkHostField, gbc);

            panel.setBorder(BorderFactory.createTitledBorder("Options"));

            // set the value from the user prefs
            networkHostField.setText(
                    SystemProperties.getProperty("user", "obex.network.host"));

            optionsPanels.put("network", panel);
            return panel;
        }

        /**
         * Builds the USB options panel.
         */
        private JPanel getUsbOptionsPanel() {
            if (optionsPanels.containsKey("usb")) {
                return optionsPanels.get("usb");
            }

            usbInterfaceField = new JTextField();

            JPanel panel = new JPanel(new GridBagLayout());
            GridBagConstraints gbc = new GridBagConstraints();
            gbc.anchor = GridBagConstraints.NORTHWEST;
            gbc.gridy = 0;
            gbc.insets.left = 5;
            gbc.insets.right = 5;
            gbc.insets.top = 5;
            panel.add(new JLabel("USB interface:"), gbc);
            gbc.gridx = 1;
            gbc.weighty = 1.0;
            gbc.weightx = 1.0;
            gbc.fill = GridBagConstraints.HORIZONTAL;
            panel.add(usbInterfaceField, gbc);

            panel.setBorder(BorderFactory.createTitledBorder("Options"));
            
            // set the value from the user prefs
            usbInterfaceField.setText(
                    SystemProperties.getProperty("user", "obex.usb.interface"));

            optionsPanels.put("usb", panel);
            return panel;
        }

        /**
         * Builds the bluetooth options panel.
         */
        private JPanel getBluetoothOptionsPanel() {
            if (optionsPanels.containsKey("bluetooth")) {
                return optionsPanels.get("bluetooth");
            }

            //bluetoothAddressField = new JTextField();
            String value = SystemProperties.getProperty(
                    "user", "obex.bluetooth.address");
            bluetoothAddressField = new JComboBox(value.split(","));
            bluetoothAddressField.setEditable(true);

            bluetoothChannelField = new JTextField();

            JPanel panel = new JPanel(new GridBagLayout());
            GridBagConstraints gbc = new GridBagConstraints();
            gbc.anchor = GridBagConstraints.NORTHWEST;
            gbc.gridy = 0;
            gbc.insets.left = 5;
            gbc.insets.right = 5;
            gbc.insets.top = 5;
            panel.add(new JLabel("Bluetooth address:"), gbc);
            gbc.gridx = 1;
            gbc.insets.right = 5;
            gbc.fill = GridBagConstraints.HORIZONTAL;
            panel.add(bluetoothAddressField, gbc);
            gbc.gridy++;
            gbc.gridx = 0;
            gbc.insets.top = 5;
            gbc.fill = GridBagConstraints.NONE;
            panel.add(new JLabel("Bluetooth channel:"), gbc);
            gbc.gridx = 1;
            gbc.weighty = 1.0;
            gbc.weightx = 1.0;
            gbc.fill = GridBagConstraints.HORIZONTAL;
            panel.add(bluetoothChannelField, gbc);

            panel.setBorder(BorderFactory.createTitledBorder("Options"));
            
            // set the value from the user prefs
            
//            bluetoothAddressField.setText(
//                    SystemProperties.getProperty("user", "obex.bluetooth.address"));
            bluetoothChannelField.setText(
                    SystemProperties.getProperty("user", "obex.channel.address"));

            optionsPanels.put("bluetooth", panel);
            return panel;
        }
        
    }
    
}



