/*
 * FolderTreePanel.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.jobexftp;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;

import org.underworldlabs.jobexftp.obexobjects.ObexFile;
import org.underworldlabs.jobexftp.obexobjects.ObexFolder;
import org.underworldlabs.jobexftp.obexobjects.ObexObject;
import org.underworldlabs.jobexftp.util.GUIUtilities;
import org.underworldlabs.swing.GUIUtils;
import org.underworldlabs.swing.tree.DynamicTree;
import org.underworldlabs.swing.util.SwingWorker;

/**
 * Folder view tree panel.
 *
 * @author Takis Diakoumis
 */
public class FolderTreePanel extends JPanel 
                             implements TreeExpansionListener,
                                        TreeSelectionListener {
    
    /** the folder tree display */
    private DynamicTree tree;

    /** node selection cache */
    private Map<ObexTreeNode,List> folderCache;
    
    /** thread worker object */
    private SwingWorker worker;

    /** obex object list comparator */
    private Comparator comparator;
    
    /** Creates a new instance of FolderTreePanel */
    public FolderTreePanel() {
        super(new BorderLayout());
        try {
            init();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void init() throws Exception {
        // init the cache
        folderCache = new HashMap<ObexTreeNode,List>();
        
        ObexObject rootObject = new ObexObject("/");
        ObexTreeNode root = new ObexTreeNode(rootObject);
        tree = new DynamicTree(root);
        tree.addTreeSelectionListener(this);
        tree.addTreeExpansionListener(this);
        tree.addMouseListener(new MouseHandler());
        
        tree.setRowHeight(20);
        tree.setCellRenderer(new ObexTreecellRenderer());

        JScrollPane scroller = new JScrollPane(tree);
        scroller.getViewport().setBackground(Color.WHITE);
        add(scroller, BorderLayout.CENTER);
        
        // hide the tree until we are connected
        tree.setVisible(false);
        
        // initialise the comparator
        comparator = new ObexObjectComparator();
        
        setPreferredSize(new Dimension(200, getHeight()));
    }

    /**
     * Indicates the currently selceted path has been refreshed.
     */
    public void selectedPathReloaded(List<ObexObject> list) {
        TreePath path = tree.getSelectionPath();
        
        DefaultMutableTreeNode selected = 
                (DefaultMutableTreeNode)path.getLastPathComponent();

        // clear the node of any children
        if (selected.getChildCount() > 0) {
            selected.removeAllChildren();
        }
        
        if (selected == tree.getRootNode()) {
            populateRootNode(list);
        }
        // must be a folder
        else {
            // remove it from the cache
            folderCache.remove(selected);
            // repopulate with the new children
            ObexTreeNode obexNode = (ObexTreeNode)selected;
            obexNode.setLoaded(false);
            ObexFolder folder = (ObexFolder)obexNode.getUserObject();
            folderSelected(obexNode, folder.getPath(), list);            
        }
    }
    
    public void connected() {
        try {
            DefaultMutableTreeNode root = tree.getRootNode();
            
            // clear the root node of any children
            if (root.getChildCount() > 0) {
                root.removeAllChildren();
            }

            List<ObexObject> list = JObexController.load(null);
            populateRootNode(list);
        }
        catch (ObexProcessException e) {
            GUIUtilities.displayErrorMessage(
                    "An error occurred populating the folder tree display:\n"
                    +e.getMessage());
        }
    }
    
    /**
     * Populates the root node with the values in the specified list.
     */
    private void populateRootNode(List<ObexObject> list) {
        DefaultMutableTreeNode root = tree.getRootNode();
        if (list != null && list.size() > 0) {
            Collections.sort(list, comparator);
            addChildren(list, root);
            tree.setVisible(true);
            tree.setSelectionRow(0);
        }
    }
    
    /**
     * Adds the specified children in the list to the specified
     * parent tree node.
     *
     * @param list - the children
     * @param parent - the parent where the children will be added
     */
    private void addChildren(List<ObexObject> list, 
                             DefaultMutableTreeNode parent) {
        if (list != null) {
            int size = list.size();
            if (size > 0) {

                String basePath = null;
                ObexFolder parentFolder = null;
                Object parentObject = parent.getUserObject();
                if (parentObject instanceof ObexObject) {
                    basePath = ((ObexObject)parentObject).getPath();
                    if (basePath.length() > 1) {
                        basePath += "/";
                    }
                }

                if (parent instanceof ObexTreeNode) {
                    ((ObexTreeNode)parent).setLoaded(true);
                }

                for (int i = 0; i < size; i++) {
                    ObexObject object = list.get(i);
                    if (object instanceof ObexFolder) {
                        String name = object.getName();
                        if (basePath != null) {
                            object.setPath(basePath+name);
                        }
                        parent.add(new ObexTreeNode(object));
                    }
                }
                tree.nodeStructureChanged(parent);

            }

        }
    }
    
    private void folderSelected(ObexTreeNode node, String path) {
        try {
            folderSelected(node, path, JObexController.load(path));
        } catch (ObexProcessException e) {
            e.printStackTrace();
        }
    }
    
    private void folderSelected(ObexTreeNode node, 
                                String path, 
                                List<ObexObject> list) {
        if (list != null) {

            Collections.sort(list, comparator);

            if (path != null) {
                String _path = path;
                if (path.length() > 1) {
                    _path += "/";
                }

                // set the path value to include the parent directory
                for (int i = 0, n = list.size(); i < n; i++) {
                    ObexObject obexObject = list.get(i);
                    String name = obexObject.getName();
                    obexObject.setPath(_path+name);
                }
            }

            // add this to the cache
            folderCache.put(node, list);

            // add relevant children to this node
            if (!node.isLoaded()) {
                addChildren(list, node);
            }

            JObexController.folderSelected(list);
        }
    }

    private void folderExpanded(ObexTreeNode node, String path) {
        try {
            List<ObexObject> list = JObexController.load(path);
            if (list != null) {
                Collections.sort(list, comparator);
            }
            addChildren(list, node);
        }
        catch (ObexProcessException e) {
            GUIUtilities.displayErrorMessage(
                    "An error occurred retrieving the values for the path " +
                    path + ".\nThe system returned:\n" +
                    e.getMessage());
        }
    }

    // --------------------------------------------------
    // ------- TreeSelectionListener implementation
    // --------------------------------------------------
    
    /**
     * Called whenever the value of the selection changes.
     * This will store the current path selection.
     *
     * @param the event that characterizes the change
     */
    public void valueChanged(TreeSelectionEvent e) {
        if (!JObexController.isConnected()) {
            return;
        }

        Object object = e.getPath().getLastPathComponent();
        if (object instanceof ObexTreeNode) {
            final ObexTreeNode node = (ObexTreeNode)object;

            // check if we already have this nodes table list
            if (folderCache.containsKey(node)) {
                JObexController.folderSelected(folderCache.get(node));
                return;
            }

            String tentativePath = null;
            if (node == tree.getRootNode()) {
                tentativePath = "";
            } else {
                Object userObject = node.getUserObject();
                if (userObject instanceof ObexFolder) {
                    tentativePath = ((ObexFolder)userObject).getPath();
                }
            }

            final String path = tentativePath;
            worker = new SwingWorker() {
                public Object construct() {
                    GUIUtilities.showWaitCursor();
                    folderSelected(node, path);
                    return "done";
                }
                public void finished() {
                    GUIUtilities.showNormalCursor();
                    GUIUtils.scheduleGC();
                }
            };
            worker.start();
        }
    }

    /**
     * Returns the current folder selection path.
     */
    protected String getSelectedFolder() {
        ObexTreeNode node = (ObexTreeNode)tree.getSelectionPath().getLastPathComponent();
        ObexObject obexObject = (ObexObject)node.getUserObject();
        return obexObject.getPath();
    }

    /**
     * Fires to expand the selected tree row - if possible.
     */
    public void selectedNodeExpanded(String name) {
        tree.expandSelectedRow();
        tree.selectNextNode(name);
    }
    
    // --------------------------------------------------
    // ------- TreeExpansionListener implementation
    // --------------------------------------------------

    public void treeExpanded(TreeExpansionEvent e) {
        TreePath treePath = e.getPath();
        Object object = treePath.getLastPathComponent();
        if (!(object instanceof DefaultMutableTreeNode)) {
            return;
        }

        DefaultMutableTreeNode parent = (DefaultMutableTreeNode)object;
        if (!(parent instanceof ObexTreeNode)) {
            return;
        }
        
        final ObexTreeNode node = (ObexTreeNode)parent;
        
        // check if we have already loaded
        if (node.isLoaded()) {
            return;
        }

        Object userObject = node.getUserObject();
        // check if its a file
        if (userObject instanceof ObexFile) {
            return;
        }

        final String path = ((ObexObject)userObject).getPath();
        
        worker = new SwingWorker() {
            public Object construct() {
                GUIUtilities.showWaitCursor();
                folderExpanded(node, path);
                return "done";
            }
            public void finished() {
                GUIUtilities.showNormalCursor();
                GUIUtils.scheduleGC();
            }
        };
        worker.start();        
    }
    
    public void treeCollapsed(TreeExpansionEvent e) {}
    
    
    class ObexTreeNode extends DefaultMutableTreeNode {

        private boolean loaded;
        
        public ObexTreeNode(ObexObject object) {
            super(object);
        }
        
        public boolean isLeaf() {
            return (getUserObject() instanceof ObexFile);
        }

        public boolean isLoaded() {
            return loaded;
        }

        public void setLoaded(boolean loaded) {
            this.loaded = loaded;
        }
        
    }
    
    class MouseHandler extends MouseAdapter {
        
        public MouseHandler() {}
        
        public void mouseClicked(MouseEvent e) {
        }

    }

    class ObexObjectComparator implements Comparator {
        
        /** <p>Compares the two objects. */
        public int compare(Object obj1, Object obj2) {
            
            if (obj1 instanceof ObexFolder) {
                
                if (obj2 instanceof ObexFolder) {
                    ObexFolder folder1 = (ObexFolder)obj1;
                    ObexFolder folder2 = (ObexFolder)obj2;
                    return folder1.getName().toUpperCase().
                            compareTo(folder2.getName().toUpperCase());
                }
                return 1;

            }
            else if (obj1 instanceof ObexFile) {
                
                if (obj2 instanceof ObexFile) {
                    ObexFile file1 = (ObexFile)obj1;
                    ObexFile file2 = (ObexFile)obj2;
                    return file1.getName().toUpperCase().
                            compareTo(file2.getName().toUpperCase());
                }
                return 1;

            }
            return 0;
        }
    }

    
    class ObexTreecellRenderer extends DefaultTreeCellRenderer {
        
        /** the folder icon */
        private ImageIcon folderIcon;
        
        // selection background and font colours
        private Color textBackground;
        private Color textForeground;
        private Color selectedBackground;
        private Color selectedTextForeground;

        public ObexTreecellRenderer() {
            folderIcon = GUIUtilities.loadIcon("Folder16.png");
            
            // selection colours
            textBackground = UIManager.getColor("Tree.textBackground");
            textForeground = UIManager.getColor("Tree.textForeground");
            selectedBackground = UIManager.getColor("Tree.selectionBackground");
            selectedTextForeground = UIManager.getColor("Tree.selectionForeground");
        }
        
        /**
         * Sets the value of the current tree cell to value. If 
         * selected is true, the cell will be drawn as if selected.
         *
         * @return the Component that the renderer uses to draw the value
         */
        public Component getTreeCellRendererComponent(JTree tree, 
                                                      Object value,
                                                      boolean isSelected, 
                                                      boolean isExpanded,
                                                      boolean isLeaf, 
                                                      int row, 
                                                      boolean hasFocus) {

            setFont(tree.getFont());
            
            // set the icon
            setIcon(folderIcon);

            // set the gap
            setIconTextGap(10);
            
            // label text for display and tip
            if (value != null) {
                String label = value.toString();
                setText(label);
                setToolTipText(label);
            } 
            else {
                setText("");
            }

            selected = isSelected;
            
            if(!isSelected) {
                setBackground(textBackground);
                setForeground(textForeground);
            } else {
                setBackground(selectedBackground);   
                setForeground(selectedTextForeground);
            }

            return this;
        }
        
    } // ObexTreecellRenderer
    
}



