/*
 * JObexController.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.jobexftp;

import java.util.List;
import org.underworldlabs.jobexftp.obexobjects.ObexFile;
import org.underworldlabs.jobexftp.obexobjects.ObexObject;
import org.underworldlabs.jobexftp.util.GUIUtilities;
import org.underworldlabs.jobexftp.util.ObexParser;
import org.underworldlabs.jobexftp.util.ObexWorker;

/**
 *
 * @author Takis Diakoumis
 */
public class JObexController {
    
    /** the base system panel */
    private static JObexBasePanel basePanel;
    
    /** the XML parser */
    private static ObexParser parser;
    
    /** the process worker utility */
    private static ObexWorker worker;

    /** Indicates a connection has been established */
    private static boolean connected;
    
    /** the last path location selected on the local file system */
    private static String lastPathLocation;
    
    /** Prevent object instantiation */
    private JObexController() {}

    protected static void folderSelected(List<ObexObject> list) {
        basePanel.folderSelected(list);
    }

    /**
     * Returns whether a command is in process.
     */
    public boolean isProcessing() {
        if (worker == null) {
            return false;
        }
        return worker.isProcessing();
    }

    public static List<ObexObject> load(String folder) 
        throws ObexProcessException {
        try {
            GUIUtilities.startProcessing();
            if (worker == null) {
                worker = new ObexWorker();
            }
            return worker.load(folder);
        }
        finally {
            GUIUtilities.stopProcessing();
        }
    }

    /**
     * Cancels the running process on the device.
     */
    public static void cancelRunningProcess() {
        if (worker != null && worker.isProcessing()) {
            worker.destroyRunningProcess();
        }
    }
    
    /**
     * Returns the currently selected folder within the tree display.
     */
    public static String getSelectedFolder() {
        return basePanel.getSelectedFolder();
    }
    
    /**
     * Creates a new directory within the specified parent directory.
     */
    public static boolean mkdir(String parent, String name) 
        throws ObexProcessException {
        try {
            GUIUtilities.startProcessing();
            if (worker == null) {
                worker = new ObexWorker();
            }
            return worker.mkdir(parent, name);
        }
        finally {
            GUIUtilities.stopProcessing();
        }
    }

    /**
     * Uploads the specified file to the current selection directory.
     *
     * @param fromPath - the path to the file to be uploaded
     * @param toPath - unimplemented at this stage (reserved)
     * @return true if the process succeeded, false otherwise
     */
    public static boolean upload(String fromPath, String toPath) 
        throws ObexProcessException {
        if (worker == null) {
            worker = new ObexWorker();
        }
        return worker.upload(fromPath, toPath);
    }

    /**
     * Deletes the specified file(s) from specified the remote path.
     *
     * @param files - the full path(s) to the file to be deleted
     * @return true if the process succeeded, false otherwise
     */
    public static boolean delete(String[] files) 
        throws ObexProcessException {
        if (worker == null) {
            worker = new ObexWorker();
        }
        for (int i = 0; i < files.length; i++) {
            worker.delete(files[i]);
        }
        // reload the current directory
        reloadSelectedPath();
        return true;
    }

    /**
     * Reloads the currently selected path.
     */
    public static void reloadSelectedPath()
        throws ObexProcessException {
        basePanel.reloadSelectedPath(load(getSelectedFolder()));
    }
    
    /**
     * Returns the currently selected folder within the tree display.
     */
    public static List<ObexFile> getSelectedFiles() {
        return basePanel.getSelectedFiles();
    }

    /**
     * Returns the selected objects (file and/or folders)
     * within the table view.
     */
    public static List<ObexObject> getSelectedObjects() {
        return basePanel.getSelectedObjects();
    }

    /**
     * Uploads the specified file to the current selection directory.
     */
    public static boolean download(String[] files, String toPath) 
        throws ObexProcessException {
        if (worker == null) {
            worker = new ObexWorker();
        }

        for (int i = 0; i < files.length; i++) {
            worker.download(files[i], toPath);
        }
        return true;
    }

    /**
     * Fires to expand the selected tree row - if possible.
     */
    public static void selectedNodeExpanded(String name) {
        basePanel.selectedNodeExpanded(name);
    }
    
    public static void registerBasePanel(JObexBasePanel _basePanel) {
        basePanel = _basePanel;
    }

    public static boolean isConnected() {
        return connected;
    }

    public static void setConnected(boolean aConnected) {
        connected = aConnected;
        if (connected) {
            basePanel.connected();
        }
    }

    public static String getLastPathLocation() {
        return lastPathLocation;
    }

    public static void setLastPathLocation(String aLastPathLocation) {
        lastPathLocation = aLastPathLocation;
    }
    
    /**
     * Returns true if the file with the specified name is 
     * a recognised image file type (tests file extension only).
     */
    public static boolean isImageFile(String name) {
        return isNameOfType(name, Constants.IMAGE_EXTENSIONS);
    }

    /**
     * Returns true if the file with the specified name is 
     * of a recognised file type as determined by the specified types
     * (tests file extension only).
     */
    public static boolean isNameOfType(String name, String[] types) {
        name = name.toUpperCase();
        for (int i = 0; i < types.length; i++) {
            if (name.endsWith(types[i])) {
                return true;
            }
        }
        return false;
    }

}



