/*
 * GUIUtilities.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.jobexftp.util;

import java.awt.Dimension;
import java.awt.Point;
import javax.swing.ActionMap;
import javax.swing.ImageIcon;
import javax.swing.InputMap;
import javax.swing.JFrame;
import org.underworldlabs.swing.GUIUtils;
import org.underworldlabs.jobexftp.JObexFTP;
import org.underworldlabs.jobexftp.StatusBarPanel;
import org.underworldlabs.swing.ExceptionErrorDialog;
import org.underworldlabs.swing.StyledLogPane;
import org.underworldlabs.swing.util.IconUtilities;
import org.underworldlabs.util.SystemProperties;

/**
 *
 * @author Takis Diakoumis
 */
public class GUIUtilities {

    /** the command output pane */
    private static StyledLogPane logPane;

    /** the status bar panel */
    private static StatusBarPanel statusBar;
    
    /** icon resource folder path */
    private static String iconFolder;
    
    private GUIUtilities() {}

    public static ImageIcon loadIcon(String fileName) {
        if (iconFolder == null) {
            iconFolder = SystemProperties.getProperty("system", "icons.path");
        }
        return IconUtilities.loadIcon(iconFolder + fileName);
    }
    
    /** <p>Retrieves the applications <code>InputMap</code>.
     *
     *  @return the <code>InputMap</code>
     */
    public static InputMap getInputMap(int condition) {
        return getApplicationFrame().getRootPane().getInputMap(condition);
    }
    
    /** <p>Retrieves the applications <code>ActionMap</code>.
     *
     *  @return the <code>ActionMap</code>
     */
    public static ActionMap getActionMap() {
        return getApplicationFrame().getRootPane().getActionMap();
    }

    /**
     * Sets the label text in the first (left-most) label of
     * the status bar.
     *
     * @param text - the text to be set
     */
    public static void setActivityText(String text) {
        statusBar.setFirstLabelText(text);
    }
    
    /**
     * Indicates that a process has begun and kicks off the
     * status bar's indeterminate progress bar.
     */
    public static void startProcessing() {
        statusBar.startProcessing();
    }

    /**
     * Indicates that a process has finished and stops the
     * status bar's indeterminate progress bar.
     */
    public static void stopProcessing() {
        statusBar.stopProcessing();
    }

    /**
     * Returns the application status bar panel.
     */
    public static StatusBarPanel getStatusBarPanel() {
        if (statusBar == null) {
            statusBar = new StatusBarPanel();
            statusBar.setThirdLabelText(
                    "JDK" + System.getProperty("java.version").substring(0,5));
        }
        return statusBar;
    }
    
    /**
     * Appends the specified text to the output log
     * pane as error text.
     */
    public static void outputErrorText(String text) {
        if (logPane == null) {
            logPane = new StyledLogPane();
        }
        logPane.appendErrorFixedWidth(text);
    }

    /**
     * Appends the specified text to the output log
     * pane as plain text.
     */
    public static void outputPlainText(String text) {
        if (logPane == null) {
            logPane = new StyledLogPane();
        }
        logPane.appendPlainFixedWidth(text);
    }

    /**
     * Returns the output log pane.
     */
    public static StyledLogPane getLogPane() {
        if (logPane == null) {
            logPane = new StyledLogPane();
        }
        return logPane;
    }

    /**
     * Returns the application frame.
     */
    public static JFrame getApplicationFrame() {
        return JObexFTP.getApplicationFrame();
    }
    
    /** <p>Calculates and returns the centered position
     *  of a dialog with the specified size to be added
     *  to the desktop area - ie. taking into account the
     *  size and location of all docked panels.
     *
     *  @param the size of the dialog to be added as a
     *         <code>Dimension</code> object
     *  @return the <code>Point</code> at which to add the dialog
     */
    public static Point getLocationForDialog(Dimension dialogDim) {        
        return GUIUtils.getLocationForDialog(
                    getApplicationFrame(), dialogDim);       
    }

    /** 
     * Sets the application cursor to the system normal cursor 
     */
    public static void showNormalCursor() {
        GUIUtils.showNormalCursor(JObexFTP.getApplicationFrame());
    }
    
    /** 
     * Sets the application cursor to the system wait cursor 
     */
    public static void showWaitCursor() {
        GUIUtils.showWaitCursor(JObexFTP.getApplicationFrame());
    }

    /**
     * Displays the error dialog displaying the stack trace from a
     * throws/caught exception.
     *
     * @param message - the error message to display
     * @param e - the throwable
     */
    public static void displayExceptionErrorDialog(String message, Throwable e) {
        new ExceptionErrorDialog(getApplicationFrame(), message, e);
    }

    // -------------------------------------------------------
    // ------ Helper methods for various option dialogs ------
    // -------------------------------------------------------
    
    // These have been revised to use JDialog as the wrapper to
    // ensure the dialog is centered within the dektop pane and not
    // within the entire screen as you get with JOptionPane.showXXX()
    
    public static final void displayInformationMessage(String message) {
        GUIUtils.displayInformationMessage(getApplicationFrame(), message);
    }
    
    public static final String displayInputMessage(String title, String message) {
        return GUIUtils.displayInputMessage(getApplicationFrame(), title, message);
    }
    
    public static final void displayWarningMessage(String message) {
        GUIUtils.displayWarningMessage(getApplicationFrame(), message);
    }
    
    public static final void displayErrorMessage(String message) {
        GUIUtils.displayErrorMessage(getApplicationFrame(), message);
    }

    public static final int displayConfirmCancelErrorMessage(String message) {
        return GUIUtils.displayConfirmCancelErrorMessage(getApplicationFrame(), message);
    }

    public static final int displayYesNoDialog(String message, String title) {
        return GUIUtils.displayYesNoDialog(getApplicationFrame(), message, title);
    }
    
    public static final int displayConfirmCancelDialog(String message) {
        return GUIUtils.displayConfirmCancelDialog(getApplicationFrame(), message);
    }
    
    public static final int displayConfirmDialog(String message) {
        return GUIUtils.displayConfirmDialog(getApplicationFrame(), message);
    }

}



