/*
 * ObexParser.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.jobexftp.util;

import java.io.CharArrayWriter;
import java.util.ArrayList;
import java.util.List;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.underworldlabs.jobexftp.obexobjects.ObexFile;
import org.underworldlabs.jobexftp.obexobjects.ObexFolder;
import org.underworldlabs.jobexftp.obexobjects.ObexObject;
import org.underworldlabs.util.MiscUtils;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * XML parser for obexftp command execution results.
 *
 * @author Takis Diakoumis
 */
public class ObexParser {

    /** XML tag name 'folder-listing' */
    private static final String FOLDER_LISTING = "folder-listing";
    
    /** XML tag name 'folder' */
    private static final String FOLDER = "folder";

    /** XML tag name 'file' */
    private static final String FILE = "file";

    /** XML tag attribute name 'name' */
    private static final String NAME = "name";
    
    /** XML tag attribute name 'size' */
    private static final String SIZE = "size";
    
    /** Creates a new instance of ObexParser */
    public ObexParser() {}
    
    public List<ObexObject> load(Process process) {
        return load(new InputSource(process.getInputStream()));
    }
    
    public List<ObexObject> load(InputSource input) {
        String userDir = System.getProperty("user.dir");
        try {
            SAXParserFactory factory = SAXParserFactory.newInstance();
            factory.setFeature("http://xml.org/sax/features/validation", false);
            factory.setFeature("http://xml.org/sax/features/external-general-entities", false);
            factory.setFeature("http://xml.org/sax/features/external-parameter-entities", false);

            // temporarily reset the property user.dir for validation
            System.setProperty("user.dir", 
                    SystemResources.getUserSettingsHomeDirectory());           
            
            SAXParser parser = factory.newSAXParser();
            ObexXMLHandler handler = new ObexXMLHandler();
            parser.parse(input, handler);
            return handler.getObexObjectList();
        }
        catch (Exception e) {
            e.printStackTrace();
            return new ArrayList<ObexObject>(0);
        }
        finally {
            // reset the user.home variable back to what it was
            System.setProperty("user.dir", userDir);
        }
    }

    
    class ObexXMLHandler extends DefaultHandler {

        /** list of obex objects retrieved */
        private List<ObexObject> list;

        /** the writer object */
        private CharArrayWriter contents;

        public ObexXMLHandler() {
            contents = new CharArrayWriter();
            list = new ArrayList<ObexObject>();
        }

        public void startElement(String nameSpaceURI, String localName,
                                 String qName, Attributes attrs) {
            contents.reset();

            String name = localName;
            if (MiscUtils.isNull(name)) {
                name = qName;
            }
            
            if (name.equals(FOLDER)) {
                ObexFolder folder = new ObexFolder();
                String value = attrs.getValue(NAME);
                if (!MiscUtils.isNull(value)) {
                    folder.setName(value);
                }
                list.add(folder);
            }
            else if (name.equals(FILE)) {
                ObexFile file = new ObexFile();
                String value = attrs.getValue(NAME);
                if (!MiscUtils.isNull(value)) {
                    file.setName(value);
                }
                value = attrs.getValue(SIZE);
                if (!MiscUtils.isNull(value)) {
                    file.setSize(Integer.parseInt(value));
                }
                list.add(file);
            }
            
        }
        
        protected List<ObexObject> getObexObjectList() {
            return list;
        }
        
        public void endElement(String nameSpaceURI, String localName,
                               String qName) {
        }
        
        public void characters(char[] data, int start, int length) {
            contents.write(data, start, length);
        }
        
        public void ignorableWhitespace(char[] data, int start, int length) {
            characters(data, start, length);
        }
        
        public void error(SAXParseException spe) throws SAXException {
            throw new SAXException(spe.getMessage());
        }
    } // XMLHandler

}



