/*
 * ObexWorker.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.jobexftp.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import org.underworldlabs.jobexftp.Constants;
import org.underworldlabs.jobexftp.obexobjects.ObexObject;
import org.underworldlabs.jobexftp.ObexProcessException;
import org.underworldlabs.util.MiscUtils;
import org.underworldlabs.util.SystemProperties;

/**
 *
 * @author Takis Diakoumis
 */
public class ObexWorker {

    /** the XML parser */
    private ObexParser parser;

    /** the current process */
    private Process process;

    /** the process stdout thread for logging */
    private StdoutThread stdout;
    
    /** the process stderr thread for logging */
    private StderrThread stderr;

    /** static variable for connection type */
    private static int connectionType;
    
    /** static variable for connection arguments */
    private static String[] connectionArgs;

    /** Creates a new instance of ObexWorker */
    public ObexWorker() {}
    
    /**
     * Destroys the current process.
     */
    public void destroyRunningProcess() {
        if (process != null) {
            process.destroy();
            process = null;
        }
    }

    /**
     * Deletes the specified file from specified the remote path.
     *
     * @param path - the full path to the file to be deleted
     * @return true if the process succeeded, false otherwise
     */
    public boolean delete(String path) 
        throws ObexProcessException {

        // build the connection argument strings
        List<String> argsList = buildConnectionArguments(
                                        connectionType, connectionArgs);

        argsList.add(SystemProperties.getProperty("system", "obexftp.delete"));
        argsList.add(path);
        
        executeObexCommand(argsList);
        int result = getProcessResult();
        if (result != 0) {
            GUIUtilities.displayErrorMessage("Error in command execution");
            return false;
        }
        return true;
    }

    /**
     * Uploads the file specified by fromPath to remote device.
     *
     * @param fromPath - the path to the file to be uploaded
     * @param toPath - unimplemented at this stage (reserved)
     * @return true if the process succeeded, false otherwise
     */
    public boolean upload(String fromPath, String toPath) 
        throws ObexProcessException {

        // build the connection argument strings
        List<String> argsList = buildConnectionArguments(
                                        connectionType, connectionArgs);

        argsList.add(SystemProperties.getProperty("system", "obexftp.put"));
        argsList.add(fromPath);
        
        executeObexCommand(argsList);
        int result = getProcessResult();
        if (result != 0) {
            GUIUtilities.displayErrorMessage("Error in command execution");
            return false;
        }
        return true;
    }

    /**
     * Downloads the file at fromPath to the directory 
     * specified by toPath.
     *
     * @return true if the process succeeded, false otherwise
     */
    public boolean download(String fromPath, String toPath) 
        throws ObexProcessException {

        // build the connection argument strings
        List<String> argsList = buildConnectionArguments(
                                        connectionType, connectionArgs);

        argsList.add(SystemProperties.getProperty("system", "obexftp.get"));
        argsList.add(fromPath);
        
        executeObexCommand(argsList, toPath);
        int result = getProcessResult();
        if (result != 0) {
            GUIUtilities.displayErrorMessage("Error in command execution");
            return false;
        }
        return true;
    }

    /**
     * Waits for the current process to terminate and returns the result.
     */
    private int getProcessResult() throws ObexProcessException {
        try {
            if (process != null) {
                return process.waitFor();
            }
            return 1;
        }
        catch (InterruptedException e) {
            throw new ObexProcessException("Process interrupted", e);
        }
        finally {
            if (process != null) {
                process.destroy();
                process = null;
            }
        }
    }
    
    public boolean mkdir(String parent, String name) 
        throws ObexProcessException {

        String newFolderName = name;
        if (parent.length() > 1) { // not the root node
            newFolderName = parent + "/" + name;
        } else {
            newFolderName = parent + name;
        }

        // build the connection argument strings
        List<String> argsList = buildConnectionArguments(
                                        connectionType, connectionArgs);

        argsList.add(SystemProperties.getProperty("system", "obexftp.mkdir"));
        argsList.add(newFolderName);
        
        executeObexCommand(argsList);
        int result = getProcessResult();
        if (result != 0) {
            GUIUtilities.displayErrorMessage("Error in command execution");
            return false;
        }
        return true;
    }
    
    public List<ObexObject> load(String folder) throws ObexProcessException {

        if (parser == null) {
            parser = new ObexParser();
        }

        // build the connection argument strings
        List<String> argsList = buildConnectionArguments(
                                        connectionType, connectionArgs);

        argsList.add(SystemProperties.getProperty("system", "obexftp.list"));

        if (folder != null) {
            argsList.add(folder);
        }
        
        // execute the command
        executeObexCommand(argsList, 
                SystemResources.getUserSettingsHomeDirectory());

        // parse the results
        List<ObexObject> list = parser.load(process);

        int result = getProcessResult();
        if (result != 0) {
            GUIUtilities.displayErrorMessage("Error in command execution");
            return null;
        }
        return list;
    }

    /**
     * Returns whether a command is in process.
     */
    public boolean isProcessing() {
        return (process != null);
    }
    
    /**
     * Creates the initial conneciton to the device.
     *
     * @return <code>Constants.CONNECTED</code> on success
     *         <code>Constants.PROCESS_ERROR</code> on failure
     */
    public Integer connect(int type, String[] args) throws ObexProcessException {

        // make sure anything running is destroyed
        destroyRunningProcess();

        // build the connection argument strings
        List<String> argsList = buildConnectionArguments(type, args);

        // execute the command
        executeObexCommand(argsList);

        try {
            // wait and retrieve the result
            int result = process.waitFor();
            if (result == 0) {
                connectionType = type;
                connectionArgs = args;
                return new Integer(Constants.CONNECTED);
            }
            return new Integer(Constants.PROCESS_ERROR);
        }
        catch (InterruptedException e) {
            if (process != null) {
                process.destroy();
            }
            process = null;
            return new Integer(Constants.PROCESS_CANCELLED);
        }
    }

    /**
     * Builds the arguments collection based on the specified connection
     * type and with the specified arguments to be passed to the command.
     */
    protected List<String> buildConnectionArguments(int type, String[] args) {
        List<String> argsList = new ArrayList<String>();
        
        // add the obexftp command string
        argsList.add(getObexFTPCommand());
        
        switch (type) {
            case Constants.IRDA_CONNECTION:
                argsList.add(SystemProperties.getProperty(
                        "system", "obexftp.irda.option"));
                break;

            case Constants.BLUETOOTH_CONNECTION:
                argsList.add(SystemProperties.getProperty(
                        "system", "obexftp.bluetooth.option"));

                // add the address option from position 0
                if (args != null && args.length >= 1) {
                    if (!MiscUtils.isNull(args[0])) {
                        argsList.add(args[0]);
                    }
                }

                // add the channel option from position 1
                if (args != null && args.length == 2) {
                    if (!MiscUtils.isNull(args[1])) {
                        argsList.add(SystemProperties.getProperty(
                                "system", "obexftp.bluetooth.channel.option"));
                        argsList.add(args[1]);
                    }
                }
                break;

            case Constants.USB_CONNECTION:
                argsList.add(SystemProperties.getProperty(
                        "system", "obexftp.usb.option"));
                
                // add the interface option from position 0
                if (args != null && args.length >= 1) {
                    if (!MiscUtils.isNull(args[0])) {
                        argsList.add(args[0]);
                    }
                }
                break;

            case Constants.SERIAL_CONNECTION:
                argsList.add(SystemProperties.getProperty(
                        "system", "obexftp.tty.option"));
                
                // add the device option from position 0
                if (args != null && args.length >= 1) {
                    if (!MiscUtils.isNull(args[0])) {
                        argsList.add(args[0]);
                    }
                }
                break;

            case Constants.NETWORK_CONNECTION:
                argsList.add(SystemProperties.getProperty(
                        "system", "obexftp.network.option"));
                
                // add the device option from position 0
                if (args != null && args.length >= 1) {
                    if (!MiscUtils.isNull(args[0])) {
                        argsList.add(args[0]);
                    }
                }
                break;

        }

        return argsList;
    }

    private void executeObexCommand(List argsList) throws ObexProcessException {
        executeObexCommand((String[])argsList.toArray(new String[argsList.size()]));
    }

    private void executeObexCommand(List argsList, String workingPath) 
        throws ObexProcessException {
        executeObexCommand(
                (String[])argsList.toArray(new String[argsList.size()]),
                workingPath);
    }

    public String getObexFTPCommand() {
        return SystemProperties.getProperty("user", "obexftp.executable");
    }

    private void executeObexCommand(String[] command) 
        throws ObexProcessException {
        executeObexCommand(command, 
                new File(SystemResources.getTempDirectoryPath()));
    }

    private void executeObexCommand(String[] command, String workingPath) 
        throws ObexProcessException {
        executeObexCommand(command, new File(workingPath));
    }

    private void executeObexCommand(String[] command, File workingDir)
        throws ObexProcessException {
        try {
            printCommand(command);
            process = Runtime.getRuntime().exec(command, null, workingDir);
            //stdout = new StdoutThread();
            stderr = new StderrThread();
        }
        catch (IOException e) {
            throw new ObexProcessException(e);
        }
    }

    private void handleException(Throwable e) throws ObexProcessException {
        throw new ObexProcessException(e);
    }
        
    private void printCommand(String[] command) {
        StringBuffer sb = new StringBuffer();
        sb.append("Executing: ");
        
        for (int i = 0; i < command.length; i++) {
            sb.append(command[i]);
            sb.append(' ');
        }
        GUIUtilities.outputPlainText(sb.toString());
    }
    
    class StdoutThread extends Thread {
        
        StdoutThread() {
            super("----thread: stout: executequery----");
            start();
        }
        
        public void run() {
            
            try {
                BufferedReader in = new BufferedReader(
                            new InputStreamReader(process.getInputStream()));

                String line;
                while((line = in.readLine()) != null) {
                    GUIUtilities.outputPlainText(line);
                }
                in.close();
            }
            catch(IOException io) {}
            catch (NullPointerException npe) {}
            
        }
        
    }
    
    class StderrThread extends Thread {
        
        StderrThread() {
            super("----thread: stderr: executequery----");
            start();
        }
        
        public void run() {
            
            try {
                
                if (process == null) {
                    return;
                }

                BufferedReader in = new BufferedReader(
                        new InputStreamReader(process.getErrorStream()));

                String line;
                while((line = in.readLine()) != null) {
                    GUIUtilities.outputPlainText(line);
                    //GUIUtilities.outputErrorText(line);
                    //append('\n' + line, errorColor);
                }                
                in.close();
                
            }            
            catch(IOException io) {}
            
        }
        
    }

}



